#include "Vista.h"
C       Here is a package for handling pixel masking.  If pixel masking
C       is not desired, use the following subroutines.

c       SUBROUTINE MASK(COL,ROW)
c       INTEGER         COL, ROW
c       RETURN
c       END
c       SUBROUTINE UNMASK(COL,ROW)
c       INTEGER          COL,ROW
c       RETURN
c       END
c       SUBROUTINE CLEARMASK
c       RETURN
c       END
c       LOGICAL FUNCTION MASKED(COL,ROW)
c       INTEGER                 COL,ROW
c       MASKED = .FALSE.
c       RETURN
c       END

C       ----------------------------------------------------------------------

        SUBROUTINE MASK(COL,ROW)

C       Mask the pixel (COL,ROW)

        INTEGER         COL, ROW

C       Include the mask.

#ifdef VMS
        INCLUDE 'VINCLUDE:MASK.INC'
#else
        INCLUDE 'vistadisk/source/include/mask.inc'
#endif

        LOGICAL GETBIT
        DATA VERBOSEMASK /.FALSE./

C       Get the word and bit for (COL,ROW).

        IF (.NOT. GETBIT(COL,ROW,IWD,IBT)) THEN 
           IF (VERBOSEMASK) THEN
              PRINT *,
     &          'WARNING: Cannot mask pixels with ROW or COL < 0 '
              PRINT *,
     &          '   or with ROW or COL > ', MAXR-1, MAXC-1, ROW, COL
           END IF
           RETURN
        END IF

C       Set the bit.
        PIXMASK(IWD) = IBSET(PIXMASK(IWD),IBT)
C       Return.

        RETURN
        END

C       ----------------------------------------------------------------------

        SUBROUTINE UNMASK(COL,ROW)

C       Unmask the pixel (COL,ROW)

        INTEGER         COL, ROW

C       Include the mask.

#ifdef VMS
        INCLUDE 'VINCLUDE:MASK.INC'
#else
        INCLUDE 'vistadisk/source/include/mask.inc'
#endif

        LOGICAL GETBIT

C       Get the word and bit for (COL,ROW).

        IF (.NOT. GETBIT(COL,ROW,IWD,IBT)) RETURN

C       Set the bit.
        PIXMASK(IWD) = IBCLR(PIXMASK(IWD),IBT)
C       Return.

        RETURN
        END

C       ---------------------------------------------------------------------

        SUBROUTINE CLEARMASK

C       Clears the mask array.

#ifdef VMS
        INCLUDE 'VINCLUDE:MASK.INC'
#else
        INCLUDE 'vistadisk/source/include/mask.inc'
#endif

        DO 8701 I=1, MASKSIZE
                PIXMASK(I) = 0
8701    CONTINUE

        RETURN
        END

C       ---------------------------------------------------------------------

        LOGICAL FUNCTION MASKED(COL,ROW)

C       Returns .TRUE. if the pixel (COL,ROW) is masked.  See the file
C       'MASK.INC' for a description of the mask array.  This routine
C       is called by various VISTA analysis routines which test for masked
C       pixels.

        INTEGER COL, ROW

C       Include the mask.

#ifdef VMS
        INCLUDE 'VINCLUDE:MASK.INC'
#else
        INCLUDE 'vistadisk/source/include/mask.inc'
#endif

        LOGICAL GETBIT, BTEST

C       Get the word and bit for (COL,ROW).

        IF (.NOT. GETBIT(COL,ROW,IWD,IBT)) THEN
	  MASKED = .FALSE.
	  RETURN
	END IF

C       Test the bit.

        MASKED = BTEST(PIXMASK(IWD),IBT)
C       Return.

        RETURN
        END

C       -----------------------------------------------------------------------

        LOGICAL FUNCTION GETBIT(COL,ROW,IWD,IBT)

C       Finds the bit which corresponds to (COL,ROW).  This bit is bit
C       IBT in MASK(IWD).

#ifdef VMS
        INCLUDE 'VINCLUDE:MASK.INC'
#else
        INCLUDE 'vistadisk/source/include/mask.inc'
#endif

        INTEGER*4       COL, ROW

C       Compute and test the bit number for (COL,ROW).

        IF (ROW .LT. 0 .OR. ROW .GT. MAXR-1  .OR.
     &      COL .LT. 0 .OR. COL .GT. MAXC-1) THEN
#ifdef VERBOSE
              PRINT *, 
     &          'WARNING: Cannot mask pixels with ROW or COL < 0 '
              PRINT *,
     &          '   or with ROW or COL > ', MAXR-1, MAXC-1, ROW, COL
#endif
                GETBIT = .FALSE.
                RETURN
        END IF

        IBIT = ROW * MAXC + COL

C       Find the word and bit in that word which corresponds to IBIT.

        IBT = MOD(IBIT,32)
        IWD = MASKSIZE - IBIT/32

C       Return.

        GETBIT = .TRUE.

        RETURN
        END
